"""
Created on Thu Feb  4 15:49:29 2021

@author: Ten Tries
"""

import argparse
from src.stream_analyzer import Stream_Analyzer
import time
import sys
import numpy as np 
import serial


port=0
while(port < 10):
    try:
        ser = serial.Serial('COM'+str(port), 1000000, timeout=3)  # open serial port
        time.sleep(1)
        devType = ser.readline().decode("utf-8")
        print(devType)
        if(devType == "RGBLEDController"):
            print("RGBLED Controller found on COM"+str(port)+"!")
            break
        ser.close()
    except:
        print("RGBLED Controller not found on COM"+str(port))
    port = port + 1

FFT_window_size_A=5
FFT_window_size_B=50
FFT_window_size_C=500
FFT_window_size_D=5000
threshold = 1
maxRatio = 10
maxSlope=500
bins=9
print(bins,0,int(bins/2),bins-1)
global lastRedFAST
global lastBlueFAST
global lastGreenFAST
global lastRedMED
global lastBlueMED
global lastGreenMED
global lastRedSLOW
global lastBlueSLOW
global lastGreenSLOW

lastRedFAST = 0.0
lastBlueFAST=0.0
lastGreenFAST=0.0
lastRedMED=0.0
lastBlueMED=0.0
lastGreenMED=0.0
lastRedSLOW=0.0
lastBlueSLOW=0.0
lastGreenSLOW=0.0

def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('--device', type=int, default=None, dest='device',
                        help='pyaudio (portaudio) device index')
    parser.add_argument('--height', type=int, default=450, dest='height',
                        help='height, in pixels, of the visualizer window')
    parser.add_argument('--n_frequency_bins', type=int, default=400, dest='frequency_bins',
                        help='The FFT features are grouped in bins')
    parser.add_argument('--verbose', action='store_true')
    parser.add_argument('--window_ratio', default='24/9', dest='window_ratio',
                        help='float ratio of the visualizer window. e.g. 24/9')
    parser.add_argument('--sleep_between_frames', dest='sleep_between_frames', action='store_true',
                        help='when true process sleeps between frames to reduce CPU usage (recommended for low update rates)')
    return parser.parse_args()

def convert_window_ratio(window_ratio):
    if '/' in window_ratio:
        dividend, divisor = window_ratio.split('/')
        try:
            float_ratio = float(dividend) / float(divisor)
        except:
            raise ValueError('window_ratio should be in the format: float/float')
        return float_ratio
    raise ValueError('window_ratio should be in the format: float/float')

def run_FFT_analyzer():
    global lastRedFAST
    global lastBlueFAST
    global lastGreenFAST
    global lastRedMED
    global lastBlueMED
    global lastGreenMED
    global lastRedSLOW
    global lastBlueSLOW
    global lastGreenSLOW

    args = parse_args()
    window_ratio = convert_window_ratio(args.window_ratio)

    earA = Stream_Analyzer(
                    device = args.device,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #device = 6,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #rate   = None,               # Audio samplerate, None uses the default source settings
                    rate   = 48000,               # Audio samplerate, None uses the default source settings
                    FFT_window_size_ms  = FFT_window_size_A,    # Window size used for the FFT transform
                    updates_per_second  = 1000,  # How often to read the audio stream for new data
                    smoothing_length_ms = 0,    # Apply some temporal smoothing to reduce noisy features
                    #n_frequency_bins = args.frequency_bins, # The FFT features are grouped in bins
                    n_frequency_bins = bins, # The FFT features are grouped in bins
                    #visualize = 1,               # Visualize the FFT features with PyGame
                    visualize = 0,               # Visualize the FFT features with PyGame
                    verbose   = args.verbose,    # Print running statistics (latency, fps, ...)
                    height    = args.height,     # Height, in pixels, of the visualizer window,
                    window_ratio = window_ratio  # Float ratio of the visualizer window. e.g. 24/9
                    )
    earB = Stream_Analyzer(
                    device = args.device,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #device = 6,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #rate   = None,               # Audio samplerate, None uses the default source settings
                    rate   = 48000,               # Audio samplerate, None uses the default source settings
                    FFT_window_size_ms  = FFT_window_size_B,    # Window size used for the FFT transform
                    updates_per_second  = 500,  # How often to read the audio stream for new data
                    smoothing_length_ms = 0,    # Apply some temporal smoothing to reduce noisy features
                    #n_frequency_bins = args.frequency_bins, # The FFT features are grouped in bins
                    n_frequency_bins = bins, # The FFT features are grouped in bins
                    #visualize = 1,               # Visualize the FFT features with PyGame
                    visualize = 0,               # Visualize the FFT features with PyGame
                    verbose   = args.verbose,    # Print running statistics (latency, fps, ...)
                    height    = args.height,     # Height, in pixels, of the visualizer window,
                    window_ratio = window_ratio  # Float ratio of the visualizer window. e.g. 24/9
                    )
    earC = Stream_Analyzer(
                    device = args.device,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #device = 6,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #rate   = None,               # Audio samplerate, None uses the default source settings
                    rate   = 48000,               # Audio samplerate, None uses the default source settings
                    FFT_window_size_ms  = FFT_window_size_C,    # Window size used for the FFT transform
                    updates_per_second  = 250,  # How often to read the audio stream for new data
                    smoothing_length_ms = 0,    # Apply some temporal smoothing to reduce noisy features
                    #n_frequency_bins = args.frequency_bins, # The FFT features are grouped in bins
                    n_frequency_bins = bins, # The FFT features are grouped in bins
                    #visualize = 1,               # Visualize the FFT features with PyGame
                    visualize = 0,               # Visualize the FFT features with PyGame
                    verbose   = args.verbose,    # Print running statistics (latency, fps, ...)
                    height    = args.height,     # Height, in pixels, of the visualizer window,
                    window_ratio = window_ratio  # Float ratio of the visualizer window. e.g. 24/9
                    )
    earD = Stream_Analyzer(
                    device = args.device,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #device = 6,        # Pyaudio (portaudio) device index, defaults to first mic input
                    #rate   = None,               # Audio samplerate, None uses the default source settings
                    rate   = 48000,               # Audio samplerate, None uses the default source settings
                    FFT_window_size_ms  = FFT_window_size_D,    # Window size used for the FFT transform
                    updates_per_second  = 125,  # How often to read the audio stream for new data
                    smoothing_length_ms = 0,    # Apply some temporal smoothing to reduce noisy features
                    #n_frequency_bins = args.frequency_bins, # The FFT features are grouped in bins
                    n_frequency_bins = bins, # The FFT features are grouped in bins
                    #visualize = 1,               # Visualize the FFT features with PyGame
                    visualize = 0,               # Visualize the FFT features with PyGame
                    verbose   = args.verbose,    # Print running statistics (latency, fps, ...)
                    height    = args.height,     # Height, in pixels, of the visualizer window,
                    window_ratio = window_ratio  # Float ratio of the visualizer window. e.g. 24/9
                    )
    fps = 60  #How often to update the FFT features + display
    last_update = time.time()
    while True:
        try:
            if (time.time() - last_update) > (1./fps):
                last_update = time.time()
                raw_fftxA, raw_fftA, binned_fftxA, binned_fftA = earA.get_audio_features()
                raw_fftxB, raw_fftB, binned_fftxB, binned_fftB = earB.get_audio_features()
                raw_fftxC, raw_fftC, binned_fftxC, binned_fftC = earC.get_audio_features()
                raw_fftxD, raw_fftD, binned_fftxD, binned_fftD = earD.get_audio_features()
                
                
                binned_fftx = binned_fftxA
                binned_fftB = np.where(binned_fftB<1, 1, binned_fftB)
                binned_fftFAST = FFT_window_size_B/FFT_window_size_A * binned_fftA / binned_fftB

                binned_fftFAST = np.where(binned_fftFAST>maxRatio+threshold, maxRatio+threshold, binned_fftFAST) 
                binned_fftFAST = np.where(binned_fftFAST<threshold, threshold, binned_fftFAST)
                binned_fftFAST = np.where(binned_fftA<10000, threshold, binned_fftFAST)
                
                #Red
                Red=int(round((binned_fftFAST[0] - threshold)*(255/maxRatio),0))
                if Red > lastRedFAST + maxSlope/FFT_window_size_A:
                    Red = int(lastRedFAST + maxSlope/FFT_window_size_A)
                    lastRedFAST = lastRedFAST + maxSlope/FFT_window_size_A
                else:
                    lastRedFAST = Red
                ser.write(('Z00r '+str(int(255-Red)).zfill(3)+'\r').encode())
                ser.write(('Z01r '+str(int(255-Red)).zfill(3)+'\r').encode())
                #if(Red>0): print(('Z00r '+str(int(255-Red)).zfill(3)+'\r'))
                #Green
                Green=int(round((binned_fftFAST[int(bins/2)] - threshold)*(255/maxRatio),0))
                if Green > lastGreenFAST + maxSlope/FFT_window_size_A:
                    Green = int(lastGreenFAST + maxSlope/FFT_window_size_A)
                    lastGreenFAST = lastGreenFAST + maxSlope/FFT_window_size_A
                else:
                    lastGreenFAST = Green
                ser.write(('Z00g '+str(int(255-Green)).zfill(3)+'\r').encode())
                ser.write(('Z01g '+str(int(255-Green)).zfill(3)+'\r').encode())
                #if(Green>0): print(('Z00g '+str(int(255-Green)).zfill(3)+'\r'))
                #Blue
                Blue=int(round((binned_fftFAST[bins-1] - threshold)*(255/maxRatio),0))
                if Blue > lastBlueFAST + maxSlope/FFT_window_size_A:
                    Blue = int(lastBlueFAST + maxSlope/FFT_window_size_A)
                    lastBlueFAST = lastBlueFAST + maxSlope/FFT_window_size_A
                else:
                    lastBlueFAST = Blue
                ser.write(('Z00b '+str(int(255-Blue)).zfill(3)+'\r').encode())
                ser.write(('Z01b '+str(int(255-Blue)).zfill(3)+'\r').encode())
                #if(Blue>0): print(('Z00b '+str(int(255-Blue)).zfill(3)+'\r'))



                binned_fftx = binned_fftxB
                binned_fftC = np.where(binned_fftC<1, 1, binned_fftC)
                binned_fftMED = FFT_window_size_C/FFT_window_size_B * binned_fftB / binned_fftC

                binned_fftMED = np.where(binned_fftMED>maxRatio+threshold, maxRatio+threshold, binned_fftMED) 
                binned_fftMED = np.where(binned_fftMED<threshold, threshold, binned_fftMED)
                binned_fftMED = np.where(binned_fftB<10000, threshold, binned_fftMED)
                #Red
                Red=int(round((binned_fftMED[0] - threshold)*(255/maxRatio),0))
                ##ser.write(('Z01r '+str(int(255-Red)).zfill(3)+'\r').encode())
                #Green
                Green=int(round((binned_fftMED[int(bins/2)] - threshold)*(255/maxRatio),0))
                ##ser.write(('Z01g '+str(int(255-Green)).zfill(3)+'\r').encode())
                #Blue
                Blue=int(round((binned_fftMED[bins-1] - threshold)*(255/maxRatio),0))
                ##ser.write(('Z01b '+str(int(255-Blue)).zfill(3)+'\r').encode())                                



                binned_fftx = binned_fftxC
                binned_fftD = np.where(binned_fftD<1, 1, binned_fftD)
                binned_fftSLOW = FFT_window_size_D/FFT_window_size_C * binned_fftC / binned_fftD

                binned_fftSLOW = np.where(binned_fftSLOW>maxRatio+threshold, maxRatio+threshold, binned_fftSLOW) 
                binned_fftSLOW = np.where(binned_fftSLOW<threshold, threshold, binned_fftSLOW)
                binned_fftSLOW = np.where(binned_fftC<10000, threshold, binned_fftSLOW)
                #Red
                Red=int(round((binned_fftSLOW[0] - threshold)*(255/maxRatio),0))
                ser.write(('Z02r '+str(int(255-Red)).zfill(3)+'\r').encode())
                #Green
                Green=int(round((binned_fftSLOW[int(bins/2)] - threshold)*(255/maxRatio),0))
                ser.write(('Z02g '+str(int(255-Green)).zfill(3)+'\r').encode())
                #Blue
                Blue=int(round((binned_fftSLOW[bins-1] - threshold)*(255/maxRatio),0))
                ser.write(('Z02b '+str(int(255-Blue)).zfill(3)+'\r').encode())

                
            elif args.sleep_between_frames:
                time.sleep(((1./fps)-(time.time()-last_update)) * 0.99)
        except KeyboardInterrupt:
            break
        except:
            print(sys.exc_info())
            pass

if __name__ == '__main__':
    try:
        run_FFT_analyzer()
    except:
        print(sys.exc_info())
        ser.close()
    ser.close()
        
